@@ -18,10 +18,6 @@ public:
     virtual int forward(const Mat& bottom_blob, Mat& top_blob, const Option& opt) const;
 
 public:
-    int expand_w;
-    int expand_h;
-    int expand_d;
-    int expand_c;
     Mat axes;
 };
 
