@@ -11,6 +11,7 @@ namespace ncnn {
 InnerProduct_vulkan::InnerProduct_vulkan()
 {
     support_vulkan = true;
+    support_vulkan_packing = true;
 
     flatten = 0;
 
@@ -130,7 +131,7 @@ int InnerProduct_vulkan::create_pipeline(const Option& _opt)
 
     size_t elemsize;
     size_t out_elemsize;
-    if (opt.use_fp16_storage || opt.use_fp16_packed)
+    if (opt.use_fp16_storage || opt.use_fp16_packed || opt.use_bf16_storage || opt.use_bf16_packed)
     {
         elemsize = in_elempack * 2u;
         out_elemsize = out_elempack * 2u;
