@@ -10,6 +10,7 @@ namespace ncnn {
 Padding_vulkan::Padding_vulkan()
 {
     support_vulkan = true;
+    support_vulkan_packing = true;
 
     pipeline_padding = 0;
     pipeline_padding_pack4 = 0;
@@ -67,7 +68,7 @@ int Padding_vulkan::create_pipeline(const Option& _opt)
 
     size_t elemsize;
     size_t out_elemsize;
-    if (opt.use_fp16_storage || opt.use_fp16_packed)
+    if (opt.use_fp16_storage || opt.use_fp16_packed || opt.use_bf16_storage || opt.use_bf16_packed)
     {
         elemsize = elempack * 2u;
         out_elemsize = out_elempack * 2u;
@@ -94,7 +95,7 @@ int Padding_vulkan::create_pipeline(const Option& _opt)
     if (one_blob_only && shape.dims != 0 && elempack > offset_elempack)
     {
         size_t offset_elemsize;
-        if (opt.use_fp16_storage || opt.use_fp16_packed)
+        if (opt.use_fp16_storage || opt.use_fp16_packed || opt.use_bf16_storage || opt.use_bf16_packed)
         {
             offset_elemsize = offset_elempack * 2u;
         }
