@@ -122,8 +122,10 @@ typedef union xmm_mm_union
 /* natural logarithm computed for 4 simultaneous float
    return NaN for x <= 0
 */
-static NCNN_FORCEINLINE v4sf log_ps(v4sf x)
+static NCNN_FORCEINLINE v4sf log_ps(const v4sf& _x)
 {
+    v4sf x = _x;
+
 #ifdef USE_SSE2
     v4si emm0;
 #else
@@ -212,8 +214,10 @@ _PS_CONST(cephes_exp_p3, 4.1665795894E-2f);
 _PS_CONST(cephes_exp_p4, 1.6666665459E-1f);
 _PS_CONST(cephes_exp_p5, 5.0000001201E-1f);
 
-static NCNN_FORCEINLINE v4sf exp_ps(v4sf x)
+static NCNN_FORCEINLINE v4sf exp_ps(const v4sf& _x)
 {
+    v4sf x = _x;
+
     v4sf tmp = _mm_setzero_ps(), fx;
 #ifdef USE_SSE2
     v4si emm0;
@@ -301,7 +305,7 @@ _PS_CONST(cephes_tanh_p8, 1.18534705686654e-04f);
 _PS_CONST(cephes_tanh_p9, 2.26843463243900e-03f);
 
 // an approximation of tanh
-static inline v4sf tanh_ps(const v4sf x)
+static inline v4sf tanh_ps(const v4sf& x)
 {
     v4sf value = x;
     value = _mm_max_ps(*(v4sf*)_ps_tanh_lo, value);
@@ -366,8 +370,11 @@ _PS_CONST(cephes_FOPI, 1.27323954473516f); // 4 / M_PI
    Since it is based on SSE intrinsics, it has to be compiled at -O2 to
    deliver full speed.
 */
-static NCNN_FORCEINLINE v4sf sin_ps(v4sf x)
-{   // any x
+static NCNN_FORCEINLINE v4sf sin_ps(const v4sf& _x)
+{
+    v4sf x = _x;
+
+    // any x
     v4sf xmm1, xmm2 = _mm_setzero_ps(), xmm3, sign_bit, y;
 
 #ifdef USE_SSE2
@@ -475,8 +482,11 @@ static NCNN_FORCEINLINE v4sf sin_ps(v4sf x)
 }
 
 /* almost the same as sin_ps */
-static NCNN_FORCEINLINE v4sf cos_ps(v4sf x)
-{   // any x
+static NCNN_FORCEINLINE v4sf cos_ps(const v4sf& _x)
+{
+    v4sf x = _x;
+
+    // any x
     v4sf xmm1, xmm2 = _mm_setzero_ps(), xmm3, y;
 #ifdef USE_SSE2
     v4si emm0, emm2;
@@ -585,8 +595,10 @@ static NCNN_FORCEINLINE v4sf cos_ps(v4sf x)
 
 /* since sin_ps and cos_ps are almost identical, sincos_ps could replace both of them..
    it is almost as fast, and gives you a free cosine with your sine */
-static NCNN_FORCEINLINE void sincos_ps(v4sf x, v4sf* s, v4sf* c)
+static NCNN_FORCEINLINE void sincos_ps(const v4sf& _x, v4sf& s, v4sf& c)
 {
+    v4sf x = _x;
+
     v4sf xmm1, xmm2, xmm3 = _mm_setzero_ps(), sign_bit_sin, y;
 #ifdef USE_SSE2
     v4si emm0, emm2, emm4;
@@ -716,15 +728,15 @@ static NCNN_FORCEINLINE void sincos_ps(v4sf x, v4sf* s, v4sf* c)
     xmm2 = _mm_add_ps(y, y2);
 
     /* update the sign */
-    *s = _mm_xor_ps(xmm1, sign_bit_sin);
-    *c = _mm_xor_ps(xmm2, sign_bit_cos);
+    s = _mm_xor_ps(xmm1, sign_bit_sin);
+    c = _mm_xor_ps(xmm2, sign_bit_cos);
 }
 
-static NCNN_FORCEINLINE __m128 tan_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 tan_ps(const __m128& x)
 {
     __m128 ysin, ycos;
     __m128 eps = _mm_set1_ps(1E-8f);
-    sincos_ps(x, &ysin, &ycos);
+    sincos_ps(x, ysin, ycos);
     __m128 mask = _mm_cmpeq_ps(ycos, _mm_setzero_ps());
     __m128 _tmp = _mm_and_ps(eps, mask);
     ycos = _mm_add_ps(ycos, _tmp);
@@ -732,13 +744,13 @@ static NCNN_FORCEINLINE __m128 tan_ps(__m128 x)
     return ytan;
 }
 
-static NCNN_FORCEINLINE __m128 pow_ps(__m128 a, __m128 b)
+static NCNN_FORCEINLINE __m128 pow_ps(const __m128& a, const __m128& b)
 {
     // pow(x, m) = exp(m * log(x))
     return exp_ps(_mm_mul_ps(b, log_ps(a)));
 }
 
-static NCNN_FORCEINLINE __m128 ceil_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 ceil_ps(const __m128& x)
 {
 #if __SSE4_1__
     return _mm_ceil_ps(x);
@@ -781,7 +793,7 @@ static NCNN_FORCEINLINE __m128 ceil_ps(__m128 x)
                _mm_andnot_ps(no_fraction, fixed_result));
 }
 
-static NCNN_FORCEINLINE __m128 floor_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 floor_ps(const __m128& x)
 {
 #if __SSE4_1__
     return _mm_floor_ps(x);
@@ -822,7 +834,7 @@ static NCNN_FORCEINLINE __m128 floor_ps(__m128 x)
                _mm_andnot_ps(no_fraction, fixed_result));
 }
 
-static NCNN_FORCEINLINE __m128 asin_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 asin_ps(const __m128& x)
 {
     const __m128 magic_negative_zero = _mm_set_ps1(-0.0f);
     const __m128 magic_half_one = _mm_set_ps1(0.5f);
@@ -905,7 +917,7 @@ static NCNN_FORCEINLINE __m128 asin_ps(__m128 x)
     return _mm_or_ps(final_approx, negative_mask);
 }
 
-static NCNN_FORCEINLINE __m128 acos_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 acos_ps(const __m128& x)
 {
     const __m128 magic_negative_zero = _mm_set_ps1(-0.0f);
     const __m128 magic_zero = _mm_set_ps1(0.0f);
@@ -994,7 +1006,7 @@ static NCNN_FORCEINLINE __m128 acos_ps(__m128 x)
                _mm_andnot_ps(is_small_input, big_final_approx));
 }
 
-static NCNN_FORCEINLINE __m128 atan_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 atan_ps(const __m128& x)
 {
     const __m128 magic_negative_zero = _mm_set_ps1(-0.0f);
     const __m128 magic_one = _mm_set_ps1(1.0f);
@@ -1086,7 +1098,7 @@ static NCNN_FORCEINLINE __m128 atan_ps(__m128 x)
                negative_mask);
 }
 
-static NCNN_FORCEINLINE __m128 atan2_ps(__m128 y, __m128 x)
+static NCNN_FORCEINLINE __m128 atan2_ps(const __m128& y, const __m128& x)
 {
     // Reference: https://mazzo.li/posts/vectorized-atan2.html
 
@@ -1148,7 +1160,7 @@ static NCNN_FORCEINLINE __m128 atan2_ps(__m128 y, __m128 x)
                _mm_andnot_ps(normal_mode, special_result));
 }
 
-static NCNN_FORCEINLINE __m128 abs_ps(__m128 x)
+static NCNN_FORCEINLINE __m128 abs_ps(const __m128& x)
 {
     const __m128 abs_mask = _mm_castsi128_ps(_mm_set1_epi32(0x7fffffff));
     return _mm_and_ps(abs_mask, x);
