@@ -45,6 +45,9 @@ public:
 #endif // NCNN_VULKAN
 
     int convert_layout(Mat& bottom_blob, const Layer* layer, const Option& opt) const;
+#if NCNN_VULKAN
+    int convert_layout(VkMat& bottom_blob, const Layer* layer, VkCompute& cmd, const Option& opt) const;
+#endif // NCNN_VULKAN
 
     int do_forward_layer(const Layer* layer, std::vector<Mat>& blob_mats, const Option& opt) const;
 #if NCNN_VULKAN
@@ -101,8 +104,9 @@ static Option get_masked_option(const Option& opt, int featmask)
     // mask option usage as layer specific featmask
     Option opt1 = opt;
     opt1.use_fp16_arithmetic = opt1.use_fp16_arithmetic && !(featmask & (1 << 0));
-    opt1.use_fp16_storage = opt1.use_fp16_storage && !(featmask & (1 << 1));
     opt1.use_fp16_packed = opt1.use_fp16_packed && !(featmask & (1 << 1));
+    opt1.use_fp16_storage = opt1.use_fp16_storage && !(featmask & (1 << 1));
+    opt1.use_bf16_packed = opt1.use_bf16_packed && !(featmask & (1 << 2));
     opt1.use_bf16_storage = opt1.use_bf16_storage && !(featmask & (1 << 2));
     opt1.use_int8_packed = opt1.use_int8_packed && !(featmask & (1 << 3));
     opt1.use_int8_storage = opt1.use_int8_storage && !(featmask & (1 << 3));
@@ -161,6 +165,9 @@ int NetPrivate::forward_layer(int layer_index, std::vector<Mat>& blob_mats, cons
 {
     const Layer* layer = layers[layer_index];
 
+    if (layer->typeindex == LayerType::Input)
+        return 0;
+
     //     NCNN_LOGE("forward_layer %d %s", layer_index, layer->name.c_str());
 
     // load bottom blobs
@@ -227,6 +234,9 @@ int NetPrivate::forward_layer(int layer_index, std::vector<Mat>& blob_mats, std:
 {
     const Layer* layer = layers[layer_index];
 
+    if (layer->typeindex == LayerType::Input)
+        return 0;
+
     //     NCNN_LOGE("forward_layer %d %d %s", layer->support_vulkan, layer_index, layer->name.c_str());
 
     bool cmd_submit_and_wait = false;
@@ -363,6 +373,9 @@ int NetPrivate::forward_layer(int layer_index, std::vector<Mat>& blob_mats, std:
 
 int NetPrivate::convert_layout(Mat& bottom_blob, const Layer* layer, const Option& opt) const
 {
+    if (bottom_blob.empty())
+        return 0;
+
     if (bottom_blob.elembits() == 32)
     {
         // clang-format off
@@ -478,6 +491,12 @@ int NetPrivate::convert_layout(Mat& bottom_blob, const Layer* layer, const Optio
                     dst_elempack = 8;
 #endif
             }
+
+            if (layer->support_any_packing)
+            {
+                // layer handles pack/unpack
+                dst_elempack = bottom_blob.elempack;
+            }
         }
     }
 
@@ -545,6 +564,46 @@ int NetPrivate::convert_layout(Mat& bottom_blob, const Layer* layer, const Optio
     return 0;
 }
 
+#if NCNN_VULKAN
+int NetPrivate::convert_layout(VkMat& bottom_blob, const Layer* layer, VkCompute& cmd, const Option& opt) const
+{
+    if (bottom_blob.empty())
+        return 0;
+
+    int dst_elempack = 1;
+    if (layer->support_vulkan_packing)
+    {
+        // resolve dst_elempack
+        int dims = bottom_blob.dims;
+        int elemcount = 0;
+        if (dims == 1) elemcount = bottom_blob.elempack * bottom_blob.w;
+        if (dims == 2) elemcount = bottom_blob.elempack * bottom_blob.h;
+        if (dims == 3 || dims == 4) elemcount = bottom_blob.elempack * bottom_blob.c;
+
+        if (elemcount % 4 == 0)
+            dst_elempack = 4;
+
+        if (layer->support_vulkan_any_packing)
+        {
+            // layer handles pack/unpack
+            dst_elempack = bottom_blob.elempack;
+        }
+    }
+
+    if (bottom_blob.elempack != dst_elempack)
+    {
+        VkMat bottom_blob_packed;
+        vkdev->convert_packing(bottom_blob, bottom_blob_packed, dst_elempack, cmd, opt);
+        bottom_blob = bottom_blob_packed;
+
+        if (bottom_blob.empty())
+            return -100;
+    }
+
+    return 0;
+}
+#endif // NCNN_VULKAN
+
 int NetPrivate::do_forward_layer(const Layer* layer, std::vector<Mat>& blob_mats, const Option& opt) const
 {
     if (layer->one_blob_only)
@@ -705,6 +764,10 @@ int NetPrivate::do_forward_layer(const Layer* layer, std::vector<VkMat>& blob_ma
             bottom_blob = bottom_blob_ref;
         }
 
+        int ret = convert_layout(bottom_blob, layer, cmd, opt);
+        if (ret != 0)
+            return ret;
+
         // forward
         if (opt.lightmode && layer->support_inplace)
         {
@@ -757,6 +820,10 @@ int NetPrivate::do_forward_layer(const Layer* layer, std::vector<VkMat>& blob_ma
             {
                 bottom_blobs[i] = bottom_blob_ref;
             }
+
+            int ret = convert_layout(bottom_blobs[i], layer, cmd, opt);
+            if (ret != 0)
+                return ret;
         }
 
         // forward
@@ -816,8 +883,11 @@ void NetPrivate::update_input_output_indexes()
     {
         if (layers[i]->typeindex == LayerType::Input)
         {
-            int blob_index = layers[i]->tops[0];
-            input_blob_indexes.push_back(blob_index);
+            for (size_t j = 0; j < layers[i]->tops.size(); j++)
+            {
+                int blob_index = layers[i]->tops[j];
+                input_blob_indexes.push_back(blob_index);
+            }
         }
     }
 
@@ -996,10 +1066,6 @@ int Net::load_param(const DataReader& dr)
     d->blobs.resize((size_t)blob_count);
 
 #if NCNN_VULKAN
-    // TODO enable gpu when bf16 conversion implemented
-    if (opt.use_bf16_storage)
-        opt.use_vulkan_compute = false;
-
     if (opt.use_vulkan_compute)
     {
         if (!d->vkdev)
@@ -1141,7 +1207,9 @@ int Net::load_param(const DataReader& dr)
         if (pdlr != 0)
         {
             NCNN_LOGE("ParamDict load_param %d %s failed", i, layer_name);
-            continue;
+            delete layer;
+            clear();
+            return -1;
         }
 
         // pull out top shape hints
@@ -1191,7 +1259,9 @@ int Net::load_param(const DataReader& dr)
         if (lr != 0)
         {
             NCNN_LOGE("layer load_param %d %s failed", i, layer_name);
-            continue;
+            delete layer;
+            clear();
+            return -1;
         }
 
         if (layer->support_int8_storage)
@@ -1217,6 +1287,7 @@ int Net::load_param(const DataReader& dr)
             if (!layer_cpu)
             {
                 NCNN_LOGE("layer %s not exists or registered", layer_type);
+                delete layer;
                 clear();
                 return -1;
             }
@@ -1233,7 +1304,10 @@ int Net::load_param(const DataReader& dr)
             if (lr != 0)
             {
                 NCNN_LOGE("layer load_param %d %s failed", i, layer_name);
-                continue;
+                delete layer;
+                delete layer_cpu;
+                clear();
+                return -1;
             }
 
             delete layer;
@@ -1295,10 +1369,6 @@ int Net::load_param_bin(const DataReader& dr)
     d->blobs.resize(blob_count);
 
 #if NCNN_VULKAN
-    // TODO enable gpu when bf16 conversion implemented
-    if (opt.use_bf16_storage)
-        opt.use_vulkan_compute = false;
-
     if (opt.use_vulkan_compute)
     {
         if (!d->vkdev)
@@ -1423,7 +1493,9 @@ int Net::load_param_bin(const DataReader& dr)
         if (pdlr != 0)
         {
             NCNN_LOGE("ParamDict load_param_bin %d failed", i);
-            continue;
+            delete layer;
+            clear();
+            return -1;
         }
 
         // pull out top blob shape hints
@@ -1473,7 +1545,9 @@ int Net::load_param_bin(const DataReader& dr)
         if (lr != 0)
         {
             NCNN_LOGE("layer load_param %d failed", i);
-            continue;
+            delete layer;
+            clear();
+            return -1;
         }
 
         if (layer->support_int8_storage)
@@ -1500,6 +1574,7 @@ int Net::load_param_bin(const DataReader& dr)
             if (!layer_cpu)
             {
                 NCNN_LOGE("layer %d not exists or registered", typeindex);
+                delete layer;
                 clear();
                 return -1;
             }
@@ -1514,7 +1589,10 @@ int Net::load_param_bin(const DataReader& dr)
             if (lr != 0)
             {
                 NCNN_LOGE("layer load_param %d failed", i);
-                continue;
+                delete layer;
+                delete layer_cpu;
+                clear();
+                return -1;
             }
 
             delete layer;
@@ -1653,6 +1731,22 @@ int Net::load_param(const char* protopath)
     fclose(fp);
     return ret;
 }
+
+#if _WIN32
+int Net::load_param(const wchar_t* protopath)
+{
+    FILE* fp = _wfopen(protopath, L"rb");
+    if (!fp)
+    {
+        NCNN_LOGE("_wfopen %ls failed", protopath);
+        return -1;
+    }
+
+    int ret = load_param(fp);
+    fclose(fp);
+    return ret;
+}
+#endif
 #endif // NCNN_STRING
 
 int Net::load_param_bin(FILE* fp)
@@ -1675,6 +1769,22 @@ int Net::load_param_bin(const char* protopath)
     return ret;
 }
 
+#if _WIN32
+int Net::load_param_bin(const wchar_t* protopath)
+{
+    FILE* fp = _wfopen(protopath, L"rb");
+    if (!fp)
+    {
+        NCNN_LOGE("_wfopen %ls failed", protopath);
+        return -1;
+    }
+
+    int ret = load_param_bin(fp);
+    fclose(fp);
+    return ret;
+}
+#endif
+
 int Net::load_model(FILE* fp)
 {
     DataReaderFromStdio dr(fp);
@@ -1694,6 +1804,22 @@ int Net::load_model(const char* modelpath)
     fclose(fp);
     return ret;
 }
+
+#if _WIN32
+int Net::load_model(const wchar_t* modelpath)
+{
+    FILE* fp = _wfopen(modelpath, L"rb");
+    if (!fp)
+    {
+        NCNN_LOGE("_wfopen %ls failed", modelpath);
+        return -1;
+    }
+
+    int ret = load_model(fp);
+    fclose(fp);
+    return ret;
+}
+#endif
 #endif // NCNN_STDIO
 
 int Net::load_param(const unsigned char* _mem)
@@ -1785,6 +1911,9 @@ void Net::clear()
     {
         Layer* layer = d->layers[i];
 
+        if (!layer)
+            continue;
+
         Option opt1 = get_masked_option(opt, layer->featmask);
 
         int dret = layer->destroy_pipeline(opt1);
